@@ -37,14 +37,24 @@ from modin.pandas.test.utils import (
     eval_general,
     generate_multiindex,
     extra_test_parameters,
+    default_to_pandas_ignore_string,
 )
 from modin.config import NPartitions
+from modin.utils import get_current_execution
+from modin.test.test_utils import warns_that_defaulting_to_pandas
 
 NPartitions.put(4)
 
 # Force matplotlib to not use any Xwindows backend.
 matplotlib.use("Agg")
 
+# Our configuration in pytest.ini requires that we explicitly catch all
+# instances of defaulting to pandas, but some test modules, like this one,
+# have too many such instances.
+# TODO(https://github.com/modin-project/modin/issues/3655): catch all instances
+# of defaulting to pandas.
+pytestmark = pytest.mark.filterwarnings(default_to_pandas_ignore_string)
+
 
 def eval_setitem(md_df, pd_df, value, col=None, loc=None):
     if loc is not None:
@@ -107,10 +117,7 @@ def test_asof_without_nan(dates, subset):
 
 @pytest.mark.parametrize(
     "lookup",
-    [
-        [60, 70, 90],
-        [60.5, 70.5, 100],
-    ],
+    [[60, 70, 90], [60.5, 70.5, 100]],
 )
 @pytest.mark.parametrize("subset", ["col2", "col1", ["col1", "col2"], None])
 def test_asof_large(lookup, subset):
@@ -184,7 +191,7 @@ def test_iloc(request, data):
     pandas_df = pandas.DataFrame(data)
 
     if not name_contains(request.node.name, ["empty_data"]):
-        # Scaler
+        # Scalar
         np.testing.assert_equal(modin_df.iloc[0, 1], pandas_df.iloc[0, 1])
 
         # Series
@@ -223,6 +230,12 @@ def test_iloc(request, data):
             modin_df.iloc[lambda df: df.index.get_indexer_for(df.index[:5])],
             pandas_df.iloc[lambda df: df.index.get_indexer_for(df.index[:5])],
         )
+
+        # Read values, selecting rows with callable and a column with a scalar.
+        df_equals(
+            pandas_df.iloc[lambda df: df.index.get_indexer_for(df.index[:5]), 0],
+            modin_df.iloc[lambda df: df.index.get_indexer_for(df.index[:5]), 0],
+        )
     else:
         with pytest.raises(IndexError):
             modin_df.iloc[0, 1]
@@ -261,6 +274,16 @@ def test_indexing_duplicate_axis(data):
     )
 
 
+@pytest.mark.parametrize("data", test_data_values, ids=test_data_keys)
+def test_set_index(data):
+    modin_df = pd.DataFrame(data)
+    pandas_df = pandas.DataFrame(data)
+
+    modin_result = modin_df.set_index([modin_df.index, modin_df.columns[0]])
+    pandas_result = pandas_df.set_index([pandas_df.index, pandas_df.columns[0]])
+    df_equals(modin_result, pandas_result)
+
+
 @pytest.mark.gpu
 @pytest.mark.parametrize("data", test_data_values, ids=test_data_keys)
 def test_keys(data):
@@ -277,13 +300,14 @@ def test_loc(data):
 
     key1 = modin_df.columns[0]
     key2 = modin_df.columns[1]
-    # Scaler
+    # Scalar
     df_equals(modin_df.loc[0, key1], pandas_df.loc[0, key1])
 
     # Series
     df_equals(modin_df.loc[0], pandas_df.loc[0])
     df_equals(modin_df.loc[1:, key1], pandas_df.loc[1:, key1])
     df_equals(modin_df.loc[1:2, key1], pandas_df.loc[1:2, key1])
+    df_equals(modin_df.loc[:, key1], pandas_df.loc[:, key1])
 
     # DataFrame
     df_equals(modin_df.loc[[1, 2]], pandas_df.loc[[1, 2]])
@@ -323,17 +347,49 @@ def test_loc(data):
     pandas_df_copy.loc[[1, 2]] = 42
     df_equals(modin_df_copy, pandas_df_copy)
 
-    # From issue #1775
-    df_equals(
-        modin_df.loc[lambda df: df.iloc[:, 0].isin(list(range(1000)))],
-        pandas_df.loc[lambda df: df.iloc[:, 0].isin(list(range(1000)))],
-    )
+    # Write an item, selecting rows with a callable.
+    modin_df_copy2 = modin_df.copy()
+    pandas_df_copy2 = pandas_df.copy()
+    modin_df_copy2.loc[lambda df: df[key1].isin(list(range(1000)))] = 42
+    pandas_df_copy2.loc[lambda df: df[key1].isin(list(range(1000)))] = 42
+    df_equals(modin_df_copy2, pandas_df_copy2)
+
+    # Write an item, selecting rows with a callable and a column with a scalar.
+    modin_df_copy3 = modin_df.copy()
+    pandas_df_copy3 = pandas_df.copy()
+    modin_df_copy3.loc[lambda df: df[key1].isin(list(range(1000))), key1] = 42
+    pandas_df_copy3.loc[lambda df: df[key1].isin(list(range(1000))), key1] = 42
+    df_equals(modin_df_copy3, pandas_df_copy3)
+
+    # Disabled for `BaseOnPython` because of the issue with `getitem_array`:
+    # https://github.com/modin-project/modin/issues/3701
+    if get_current_execution() != "BaseOnPython":
+        # From issue #1775
+        df_equals(
+            modin_df.loc[lambda df: df.iloc[:, 0].isin(list(range(1000)))],
+            pandas_df.loc[lambda df: df.iloc[:, 0].isin(list(range(1000)))],
+        )
+
+        # Read values, selecting rows with a callable and a column with a scalar.
+        df_equals(
+            pandas_df.loc[lambda df: df[key1].isin(list(range(1000))), key1],
+            modin_df.loc[lambda df: df[key1].isin(list(range(1000))), key1],
+        )
 
     # From issue #1374
     with pytest.raises(KeyError):
         modin_df.loc["NO_EXIST"]
 
 
+# This tests the bug from https://github.com/modin-project/modin/issues/3736
+def test_loc_setting_single_categorical_column():
+    modin_df = pd.DataFrame({"status": ["a", "b", "c"]}, dtype="category")
+    pandas_df = pandas.DataFrame({"status": ["a", "b", "c"]}, dtype="category")
+    modin_df.loc[1:3, "status"] = "a"
+    pandas_df.loc[1:3, "status"] = "a"
+    df_equals(modin_df, pandas_df)
+
+
 def test_loc_multi_index():
     modin_df = pd.read_csv(
         "modin/pandas/test/data/blah.csv", header=[0, 1, 2, 3], index_col=0
@@ -406,6 +462,16 @@ def test_loc_multi_index():
     df_equals(modin_df.loc[modin_df.index[:7]], pandas_df.loc[pandas_df.index[:7]])
 
 
+def test_loc_empty():
+    pandas_df = pandas.DataFrame(index=range(5))
+    modin_df = pd.DataFrame(index=range(5))
+
+    df_equals(pandas_df.loc[1], modin_df.loc[1])
+    pandas_df.loc[1] = 3
+    modin_df.loc[1] = 3
+    df_equals(pandas_df, modin_df)
+
+
 @pytest.mark.parametrize("index", [["row1", "row2", "row3"]])
 @pytest.mark.parametrize("columns", [["col1", "col2"]])
 def test_loc_assignment(index, columns):
@@ -485,15 +551,16 @@ def test_iloc_assignment():
     modin_df.iloc[0]["col1"] = 11
     modin_df.iloc[1]["col1"] = 21
     modin_df.iloc[2]["col1"] = 31
-    modin_df.iloc[0]["col2"] = 12
-    modin_df.iloc[1]["col2"] = 22
-    modin_df.iloc[2]["col2"] = 32
+    modin_df.iloc[lambda df: 0]["col2"] = 12
+    modin_df.iloc[1][lambda df: ["col2"]] = 22
+    modin_df.iloc[lambda df: 2][lambda df: ["col2"]] = 32
     pandas_df.iloc[0]["col1"] = 11
     pandas_df.iloc[1]["col1"] = 21
     pandas_df.iloc[2]["col1"] = 31
-    pandas_df.iloc[0]["col2"] = 12
-    pandas_df.iloc[1]["col2"] = 22
-    pandas_df.iloc[2]["col2"] = 32
+    pandas_df.iloc[lambda df: 0]["col2"] = 12
+    pandas_df.iloc[1][lambda df: ["col2"]] = 22
+    pandas_df.iloc[lambda df: 2][lambda df: ["col2"]] = 32
+
     df_equals(modin_df, pandas_df)
 
 
@@ -513,6 +580,16 @@ def test_iloc_nested_assignment(data):
     df_equals(modin_df, pandas_df)
 
 
+def test_iloc_empty():
+    pandas_df = pandas.DataFrame(index=range(5))
+    modin_df = pd.DataFrame(index=range(5))
+
+    df_equals(pandas_df.iloc[1], modin_df.iloc[1])
+    pandas_df.iloc[1] = 3
+    modin_df.iloc[1] = 3
+    df_equals(pandas_df, modin_df)
+
+
 def test_loc_series():
     md_df, pd_df = create_test_dfs({"a": [1, 2], "b": [3, 4]})
 
@@ -522,6 +599,29 @@ def test_loc_series():
     df_equals(pd_df, md_df)
 
 
+@pytest.mark.parametrize("locator_name", ["loc", "iloc"])
+@pytest.mark.parametrize(
+    "slice_indexer",
+    [
+        slice(None, None, -2),
+        slice(1, 10, None),
+        slice(None, 10, None),
+        slice(10, None, None),
+        slice(10, None, -2),
+        slice(-10, None, -2),
+        slice(None, 1_000_000_000, None),
+    ],
+)
+def test_loc_iloc_slice_indexer(locator_name, slice_indexer):
+    md_df, pd_df = create_test_dfs(test_data_values[0])
+    # Shifting the index, so labels won't match its position
+    shifted_index = pandas.RangeIndex(1, len(md_df) + 1)
+    md_df.index = shifted_index
+    pd_df.index = shifted_index
+
+    eval_general(md_df, pd_df, lambda df: getattr(df, locator_name)[slice_indexer])
+
+
 @pytest.mark.parametrize("data", test_data_values, ids=test_data_keys)
 def test_pop(request, data):
     modin_df = pd.DataFrame(data)
@@ -584,7 +684,7 @@ def test_reindex_like():
         columns=["temp_celsius", "windspeed"],
         index=pd.DatetimeIndex(["2014-02-12", "2014-02-13", "2014-02-15"]),
     )
-    with pytest.warns(UserWarning):
+    with warns_that_defaulting_to_pandas():
         df2.reindex_like(df1)
 
 
@@ -1329,7 +1429,7 @@ def test_xs():
     }
     df = pd.DataFrame(data=d)
     df = df.set_index(["class", "animal", "locomotion"])
-    with pytest.warns(UserWarning):
+    with warns_that_defaulting_to_pandas():
         df.xs("mammal")
 
 
@@ -1354,6 +1454,8 @@ def test___getitem__(data):
         (1, -1),
         (-3, -1),
         (1, -1, 2),
+        (-1, 1, -1),
+        (None, None, 2),
     ]
 
     # slice test
@@ -1616,6 +1718,36 @@ def test_setitem_on_empty_df(data, value, convert_to_series, new_col_id):
     eval_general(modin_df, pandas_df, applyier)
 
 
+def test___setitem__unhashable_list():
+    # from #3258 and #3291
+    cols = ["a", "b"]
+    modin_df = pd.DataFrame([[0, 0]], columns=cols)
+    modin_df[cols] = modin_df[cols]
+    pandas_df = pandas.DataFrame([[0, 0]], columns=cols)
+    pandas_df[cols] = pandas_df[cols]
+    df_equals(modin_df, pandas_df)
+
+
+def test___setitem__single_item_in_series():
+    # Test assigning a single item in a Series for issue
+    # https://github.com/modin-project/modin/issues/3860
+    modin_series = pd.Series(99)
+    pandas_series = pandas.Series(99)
+    modin_series[:1] = pd.Series(100)
+    pandas_series[:1] = pandas.Series(100)
+    df_equals(modin_series, pandas_series)
+
+
+def test___setitem__assigning_single_categorical_sets_correct_dtypes():
+    # This test case comes from
+    # https://github.com/modin-project/modin/issues/3895
+    modin_df = pd.DataFrame({"categories": ["A"]})
+    modin_df["categories"] = pd.Categorical(["A"])
+    pandas_df = pandas.DataFrame({"categories": ["A"]})
+    pandas_df["categories"] = pandas.Categorical(["A"])
+    df_equals(modin_df, pandas_df)
+
+
 @pytest.mark.parametrize("data", test_data_values, ids=test_data_keys)
 def test___len__(data):
     modin_df = pd.DataFrame(data)
